"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SqliteAdapter = void 0;
const tslib_1 = require("tslib");
const FS = tslib_1.__importStar(require("fs"));
const Path = tslib_1.__importStar(require("path"));
const sqlite_1 = tslib_1.__importDefault(require("sqlite"));
const sqlite3_1 = tslib_1.__importDefault(require("sqlite3"));
class SqliteAdapter {
    constructor() {
        this.type = 'sqlite';
    }
    async runScript({ cwd, definition, parameters, options, resourcesPath: resourcePath, resourcesBaseURL: resourceBaseURL, onOutput, }) {
        try {
            if (!options || !options.path) {
                return {
                    result: 'options-error',
                    message: 'The "path" field is not found in options.',
                };
            }
            let db = await sqlite_1.default.open({
                filename: options.path,
                driver: sqlite3_1.default.Database,
            });
            let buffer = await FS.promises.readFile(Path.join(cwd, definition.file));
            let result = await db.run(buffer.toString(), {
                $resource_path: resourcePath,
                $resource_base_url: resourceBaseURL,
                ...Object.fromEntries(Object.entries(parameters).map(([key, value]) => [`$${key}`, value])),
            });
            onOutput(`本次执行影响了 ${result.changes} 条数据, 最后一条数据 Id 为 ${result.lastID}`);
            return {
                result: 'done',
                message: '',
            };
        }
        catch (error) {
            return { result: 'unknown-error', message: error.message || String(error) };
        }
    }
}
exports.SqliteAdapter = SqliteAdapter;
//# sourceMappingURL=sqlite-adapter.js.map