"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SocketService = void 0;
const tslib_1 = require("tslib");
const url_1 = tslib_1.__importDefault(require("url"));
const socket_io_client_1 = tslib_1.__importDefault(require("socket.io-client"));
const shared_1 = require("../shared");
class SocketService {
    constructor(config, entrancesReady) {
        var _a;
        this.config = config;
        this.entrancesReady = entrancesReady;
        this.registered = false;
        let makescriptSecretURL = this.config.makescript.joinLink;
        let url = url_1.default.parse(makescriptSecretURL);
        this.socket = socket_io_client_1.default.io(`${url.protocol}//${url.host}`, {
            path: (_a = url.pathname) !== null && _a !== void 0 ? _a : '/',
        });
        let socket = this.socket;
        this.makescriptRPC = shared_1.wrapSocketToRPC(socket, shared_1.logger);
        socket.on('connect', () => {
            shared_1.logger.info(`Connected to ${makescriptSecretURL}`);
            (async () => {
                await this.entrancesReady;
                await this.makescriptRPC.register(this.config.makescript.namespace, this.registered);
                this.registered = true;
                shared_1.logger.info(`Successfully to registered as "${this.config.makescript.namespace}"`);
            })().catch(shared_1.logger.error);
        });
        socket.on('disconnect', (reason) => shared_1.logger.error(`Disconnected from ${makescriptSecretURL}: ${reason}`));
        socket.on('connect_error', (error) => shared_1.logger.error(`Failed to connect to ${makescriptSecretURL}: ${error.message}`));
    }
}
exports.SocketService = SocketService;
//# sourceMappingURL=socket-service.js.map