"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.convertEventNameToRPCMethodName = exports.convertRPCMethodNameToEventName = exports.wrapSocketToRPC = exports.bridgeRPC = void 0;
function bridgeRPC(rpc, socket, logger = console) {
    socket.onAny((event, { parameters }, callback) => {
        let methodName = convertEventNameToRPCMethodName(event);
        if (!methodName) {
            return;
        }
        if (!(methodName in rpc)) {
            logger.error(`Called an unknown RPC method "${methodName}"`);
            return;
        }
        if (typeof rpc[methodName] !== 'function') {
            logger.error(`RPC method "${methodName}" is not a callable method`);
            return;
        }
        Promise.resolve(rpc[methodName](...parameters))
            .then(result => callback({ result }))
            .catch(error => { var _a; return callback({ error: (_a = error.message) !== null && _a !== void 0 ? _a : 'Unknown error' }); });
    });
}
exports.bridgeRPC = bridgeRPC;
function wrapSocketToRPC(socket, logger = console) {
    return new Proxy({}, {
        get(_, methodName) {
            if (typeof methodName !== 'string') {
                throw new Error(`Unknown RPC call "${String(methodName)}"`);
            }
            return async (...params) => {
                return new Promise((resolve, reject) => {
                    socket.emit(convertRPCMethodNameToEventName(methodName), { parameters: params }, ({ error, result }) => {
                        if (error) {
                            logger.error(`RPC calling thrown an error: ${error}`);
                            reject(new Error(error));
                        }
                        else {
                            resolve(result);
                        }
                    });
                });
            };
        },
    });
}
exports.wrapSocketToRPC = wrapSocketToRPC;
function convertRPCMethodNameToEventName(name) {
    return `rpc:${name}`;
}
exports.convertRPCMethodNameToEventName = convertRPCMethodNameToEventName;
function convertEventNameToRPCMethodName(name) {
    let execResult = /^rpc:(.+)$/.exec(name);
    if (!execResult) {
        return undefined;
    }
    return execResult[1];
}
exports.convertEventNameToRPCMethodName = convertEventNameToRPCMethodName;
//# sourceMappingURL=rpc.js.map